@@ -30,7 +30,8 @@ public class I2PTunnelIRCClient extends I2PTunnelClientBase {
 
     /** list of Destination objects that we point at */
     private final List<I2PSocketAddress> _addrs;
-    private static final long DEFAULT_READ_TIMEOUT = 5*60*1000; // -1
+    // application should ping timeout before this
+    private static final long DEFAULT_READ_TIMEOUT = 10*60*1000;
     protected long readTimeout = DEFAULT_READ_TIMEOUT;
     private final boolean _dccEnabled;
     private I2PTunnelDCCServer _DCCServer;
