@@ -604,7 +604,7 @@ public class TunnelConfig {
             if (TunnelController.TYPE_HTTP_CLIENT.equals(_type))
                 _booleanOptions.add(I2PTunnelHTTPClient.PROP_SSL_SET);
             for (String p : _booleanClientOpts)
-                config.setProperty(OPT + p, "" + _booleanOptions.contains(p));
+                config.setProperty(OPT + p, Boolean.toString(_booleanOptions.contains(p)));
             for (String p : _otherClientOpts) {
                 if (_otherOptions.containsKey(p))
                     config.setProperty(OPT + p, _otherOptions.get(p));
@@ -616,7 +616,7 @@ public class TunnelConfig {
             // see TunnelController.setConfig()
             _booleanOptions.add(TunnelController.PROP_LIMITS_SET);
             for (String p : _booleanServerOpts)
-                config.setProperty(OPT + p, "" + _booleanOptions.contains(p));
+                config.setProperty(OPT + p, Boolean.toString(_booleanOptions.contains(p)));
             for (String p : _otherServerOpts) {
                 if (_otherOptions.containsKey(p))
                     config.setProperty(OPT + p, _otherOptions.get(p));
@@ -634,7 +634,7 @@ public class TunnelConfig {
         if (TunnelController.TYPE_HTTP_CLIENT.equals(_type) || TunnelController.TYPE_CONNECT.equals(_type) || 
             TunnelController.TYPE_SOCKS.equals(_type) ||TunnelController.TYPE_SOCKS_IRC.equals(_type)) {
             for (String p : _booleanProxyOpts)
-                config.setProperty(OPT + p, "" + _booleanOptions.contains(p));
+                config.setProperty(OPT + p, Boolean.toString(_booleanOptions.contains(p)));
             if (_proxyList != null)
                 config.setProperty(TunnelController.PROP_PROXIES, _proxyList);
         }
@@ -705,7 +705,7 @@ public class TunnelConfig {
         if (TunnelController.TYPE_IRC_CLIENT.equals(_type)) {
             boolean dcc = _booleanOptions.contains(I2PTunnelIRCClient.PROP_DCC);
             config.setProperty(OPT + I2PTunnelIRCClient.PROP_DCC,
-                               "" + dcc);
+                               Boolean.toString(dcc));
             // add some sane server options since they aren't in the GUI (yet)
             if (dcc) {
                 config.setProperty(OPT + PROP_MAX_CONNS_MIN, "3");
@@ -811,8 +811,8 @@ public class TunnelConfig {
         "proxyUsername", "proxyPassword"
         };
 
-    public static final Set<String> _noShowSet = new HashSet<String>(128);
-    public static final Set<String> _nonProxyNoShowSet = new HashSet<String>(4);
+    static final Set<String> _noShowSet = new HashSet<String>(128);
+    static final Set<String> _nonProxyNoShowSet = new HashSet<String>(4);
     static {
         _noShowSet.addAll(Arrays.asList(_noShowOpts));
         _noShowSet.addAll(Arrays.asList(_booleanClientOpts));
@@ -836,7 +836,7 @@ public class TunnelConfig {
             if ( (_i2cpPort != null) && (_i2cpPort.trim().length() > 0) ) {
                 config.setProperty(TunnelController.PROP_I2CP_PORT, _i2cpPort);
             } else {
-                config.setProperty(TunnelController.PROP_I2CP_PORT, "7654");
+                config.setProperty(TunnelController.PROP_I2CP_PORT, Integer.toString(I2PClient.DEFAULT_LISTEN_PORT));
             }
         }
         if (_privKeyFile != null)
