@@ -14,6 +14,7 @@ import java.util.List;
 import java.util.Set;
 
 import net.i2p.I2PException;
+import net.i2p.client.I2PClient;
 import net.i2p.crypto.SigType;
 import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
@@ -61,7 +62,7 @@ public class EditBean extends IndexBean {
 
     public String getTargetPort(int tunnel) {
         int port = _helper.getTargetPort(tunnel);
-        return port > 0 ? "" + port : "";
+        return port > 0 ? Integer.toString(port) : "";
     }
 
     public String getPrivateKeyFile(int tunnel) {
@@ -446,7 +447,7 @@ public class EditBean extends IndexBean {
         if (tun != null)
             return tun.getI2CPPort();
         else
-            return "7654";
+            return Integer.toString(I2PClient.DEFAULT_LISTEN_PORT);
     }
 
     public String getCustomOptions(int tunnel) {
