@@ -1,5 +1,6 @@
 package net.i2p.router.web;
 
+import java.io.Writer;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -18,6 +19,8 @@ import net.i2p.util.Log;
  * The form is "processed" after the properties are set and the first output
  * property is retrieved - either getAll(), getNotices() or getErrors().
  *
+ * This Handler will only process a single POST. The jsp bean must be declared scope=request.
+ *
  */
 public abstract class FormHandler {
     protected RouterContext _context;
@@ -34,6 +37,7 @@ public abstract class FormHandler {
     private final List<String> _notices;
     private boolean _processed;
     private boolean _valid;
+    protected Writer _out;
     
     public FormHandler() {
         _errors = new ArrayList<String>();
@@ -89,6 +93,7 @@ public abstract class FormHandler {
      * Curses Jetty for returning arrays.
      *
      * @since 0.9.4 consolidated from numerous FormHandlers
+     * @return trimmed string or null
      */
     protected String getJettyString(String key) {
         if (_settings == null)
@@ -107,6 +112,11 @@ public abstract class FormHandler {
      */
     public void storeMethod(String val) { _method = val; }
 
+    /**
+     * @since 0.9.38
+     */
+    public void storeWriter(Writer out) { _out = out; }
+
     /**
      * The old nonces from the session
      * @since 0.9.4
@@ -117,11 +127,12 @@ public abstract class FormHandler {
     }
     
     /**
-     * Override this to perform the final processing (in turn, adding formNotice
+     * Implement this to perform the final processing (in turn, adding formNotice
      * and formError messages, etc)
      *
+     * Will only be called if _action is non-null and the nonce is valid.
      */
-    protected void processForm() {}
+    protected abstract void processForm();
     
     /**
      * Add an error message to display
