@@ -26,17 +26,20 @@ import java.util.Set;
 import java.util.TreeSet;
 
 import net.i2p.crypto.SigType;
+import net.i2p.data.DatabaseEntry;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.Hash;
 import net.i2p.data.Lease;
 import net.i2p.data.LeaseSet;
+import net.i2p.data.LeaseSet2;
 import net.i2p.data.router.RouterAddress;
 import net.i2p.data.router.RouterInfo;
 import net.i2p.router.RouterContext;
 import net.i2p.router.TunnelPoolSettings;
 import net.i2p.router.util.HashDistance;   // debug
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
+import static net.i2p.router.sybil.Util.biLog2;
 import net.i2p.router.web.Messages;
 import net.i2p.router.web.WebAppStarter;
 import net.i2p.util.Log;
@@ -86,20 +89,61 @@ class NetDbRenderer {
     /**
      *  One String must be non-null
      *
+     *  @param page zero-based
      *  @param routerPrefix may be null. "." for our router only
      *  @param version may be null
      *  @param country may be null
      *  @param family may be null
      */
-    public void renderRouterInfoHTML(Writer out, String routerPrefix, String version,
+    public void renderRouterInfoHTML(Writer out, int pageSize, int page,
+                                     String routerPrefix, String version,
                                      String country, String family, String caps,
                                      String ip, String sybil, int port, SigType type,
-                                     String mtu, String ipv6, String ssucaps, int cost) throws IOException {
+                                     String mtu, String ipv6, String ssucaps,
+                                     String tr, int cost) throws IOException {
         StringBuilder buf = new StringBuilder(4*1024);
         List<Hash> sybils = sybil != null ? new ArrayList<Hash>(128) : null;
         if (".".equals(routerPrefix)) {
             renderRouterInfo(buf, _context.router().getRouterInfo(), true, true);
         } else {
+            StringBuilder ubuf = new StringBuilder();
+            if (routerPrefix != null)
+                ubuf.append("&amp;r=").append(routerPrefix);
+            if (version != null)
+                ubuf.append("&amp;v=").append(version);
+            if (country != null)
+                ubuf.append("&amp;c=").append(country);
+            if (family != null)
+                ubuf.append("&amp;fam=").append(family);
+            if (caps != null)
+                ubuf.append("&amp;caps=").append(caps);
+            if (tr != null)
+                ubuf.append("&amp;tr=").append(tr);
+            if (type != null)
+                ubuf.append("&amp;type=").append(type);
+            if (ip != null)
+                ubuf.append("&amp;ip=").append(ip);
+            if (port != 0)
+                ubuf.append("&amp;port=").append(port);
+            if (mtu != null)
+                ubuf.append("&amp;mtu=").append(mtu);
+            if (ipv6 != null)
+                ubuf.append("&amp;ipv6=").append(ipv6);
+            if (ssucaps != null)
+                ubuf.append("&amp;ssucaps=").append(ssucaps);
+            if (cost != 0)
+                ubuf.append("&amp;cost=").append(cost);
+            if (sybil != null)
+                ubuf.append("&amp;sybil=").append(sybil);
+            if (page > 0) {
+                buf.append("<div class=\"netdbnotfound\">" +
+                           "<a href=\"/netdb?pg=").append(page)
+                   .append("&amp;ps=").append(pageSize).append(ubuf).append("\">");
+                buf.append(_t("Previous Page"));
+                buf.append("</a>&nbsp;&nbsp;&nbsp;");
+                buf.append(_t("Page")).append(' ').append(page + 1);
+                buf.append("</div>");
+            }
             boolean notFound = true;
             Set<RouterInfo> routers = _context.netDb().getRouters();
             int ipMode = 0;
@@ -110,21 +154,40 @@ class NetDbRenderer {
                     ipMode = 2;
                 } else if (ip.endsWith("/8")) {
                     ipMode = 3;
+                } else if (ip.indexOf(':') > 0) {
+                    ipMode = 4;
                 }
-                for (int i = 0; i < ipMode; i++) {
-                    int last = ip.substring(0, ip.length() - 1).lastIndexOf('.');
-                    if (last > 0)
-                        ip = ip.substring(0, last + 1);
+                if (ipMode > 0 && ipMode < 4) {
+                    for (int i = 0; i < ipMode; i++) {
+                        int last = ip.substring(0, ip.length() - 1).lastIndexOf('.');
+                        if (last > 0)
+                            ip = ip.substring(0, last + 1);
+                    }
                 }
             }
+            int toSkip = pageSize * page;
+            int skipped = 0;
+            int written = 0;
+            boolean morePages = false;
+            outerloop:
             for (RouterInfo ri : routers) {
                 Hash key = ri.getIdentity().getHash();
                 if ((routerPrefix != null && key.toBase64().startsWith(routerPrefix)) ||
                     (version != null && version.equals(ri.getVersion())) ||
                     (country != null && country.equals(_context.commSystem().getCountry(key))) ||
                     (family != null && family.equals(ri.getOption("family"))) ||
-                    (caps != null && ri.getCapabilities().contains(caps)) ||
+                    // 'O' will catch PO and XO also
+                    (caps != null && hasCap(ri, caps)) ||
+                    (tr != null && ri.getTargetAddress(tr) != null) ||
                     (type != null && type == ri.getIdentity().getSigType())) {
+                    if (skipped < toSkip) {
+                        skipped++;
+                        continue;
+                    }
+                    if (written++ >= pageSize) {
+                        morePages = true;
+                        break;
+                    }
                     renderRouterInfo(buf, ri, false, true);
                     if (sybil != null)
                         sybils.add(key);
@@ -133,6 +196,14 @@ class NetDbRenderer {
                     for (RouterAddress ra : ri.getAddresses()) {
                         if (ipMode == 0) {
                             if (ip.equals(ra.getHost())) {
+                                if (skipped < toSkip) {
+                                    skipped++;
+                                    break;
+                                }
+                                if (written++ >= pageSize) {
+                                    morePages = true;
+                                    break outerloop;
+                                }
                                 renderRouterInfo(buf, ri, false, true);
                                 if (sybil != null)
                                     sybils.add(key);
@@ -142,6 +213,14 @@ class NetDbRenderer {
                         } else {
                             String host = ra.getHost();
                             if (host != null && host.startsWith(ip)) {
+                                if (skipped < toSkip) {
+                                    skipped++;
+                                    break;
+                                }
+                                if (written++ >= pageSize) {
+                                    morePages = true;
+                                    break outerloop;
+                                }
                                 renderRouterInfo(buf, ri, false, true);
                                 if (sybil != null)
                                     sybils.add(key);
@@ -153,6 +232,14 @@ class NetDbRenderer {
                 } else if (port != 0) {
                     for (RouterAddress ra : ri.getAddresses()) {
                         if (port == ra.getPort()) {
+                            if (skipped < toSkip) {
+                                skipped++;
+                                break;
+                            }
+                            if (written++ >= pageSize) {
+                                morePages = true;
+                                break outerloop;
+                            }
                             renderRouterInfo(buf, ri, false, true);
                             if (sybil != null)
                                 sybils.add(key);
@@ -163,6 +250,14 @@ class NetDbRenderer {
                 } else if (mtu != null) {
                     for (RouterAddress ra : ri.getAddresses()) {
                         if (mtu.equals(ra.getOption("mtu"))) {
+                            if (skipped < toSkip) {
+                                skipped++;
+                                break;
+                            }
+                            if (written++ >= pageSize) {
+                                morePages = true;
+                                break outerloop;
+                            }
                             renderRouterInfo(buf, ri, false, true);
                             if (sybil != null)
                                 sybils.add(key);
@@ -174,6 +269,14 @@ class NetDbRenderer {
                     for (RouterAddress ra : ri.getAddresses()) {
                         String host = ra.getHost();
                         if (host != null && host.startsWith(ipv6)) {
+                            if (skipped < toSkip) {
+                                skipped++;
+                                break;
+                            }
+                            if (written++ >= pageSize) {
+                                morePages = true;
+                                break outerloop;
+                            }
                             renderRouterInfo(buf, ri, false, true);
                             if (sybil != null)
                                 sybils.add(key);
@@ -186,6 +289,14 @@ class NetDbRenderer {
                         if (!"SSU".equals(ra.getTransportStyle()))
                             continue;
                         if (ssucaps.equals(ra.getOption("caps"))) {
+                            if (skipped < toSkip) {
+                                skipped++;
+                                break;
+                            }
+                            if (written++ >= pageSize) {
+                                morePages = true;
+                                break outerloop;
+                            }
                             renderRouterInfo(buf, ri, false, true);
                             if (sybil != null)
                                 sybils.add(key);
@@ -196,6 +307,14 @@ class NetDbRenderer {
                 } else if (cost != 0) {
                     for (RouterAddress ra : ri.getAddresses()) {
                         if (cost == ra.getCost()) {
+                            if (skipped < toSkip) {
+                                skipped++;
+                                break;
+                            }
+                            if (written++ >= pageSize) {
+                                morePages = true;
+                                break outerloop;
+                            }
                             renderRouterInfo(buf, ri, false, true);
                             if (sybil != null)
                                 sybils.add(key);
@@ -218,6 +337,22 @@ class NetDbRenderer {
                     buf.append(_t("Family")).append(' ').append(family);
                 buf.append(' ').append(_t("not found in network database"));
                 buf.append("</div>");
+            } else if (page > 0 || morePages) {
+                buf.append("<div class=\"netdbnotfound\">");
+                if (page > 0) {
+                    buf.append("<a href=\"/netdb?pg=").append(page)
+                       .append("&amp;ps=").append(pageSize).append(ubuf).append("\">");
+                    buf.append(_t("Previous Page"));
+                    buf.append("</a>&nbsp;&nbsp;&nbsp;");
+                }
+                buf.append(_t("Page")).append(' ').append(page + 1);
+                if (morePages) {
+                    buf.append("&nbsp;&nbsp;&nbsp;<a href=\"/netdb?pg=").append(page + 2)
+                       .append("&amp;ps=").append(pageSize).append(ubuf).append("\">");
+                    buf.append(_t("Next Page"));
+                    buf.append("</a>");
+                }
+                buf.append("</div>");
             }
         }
         out.write(buf.toString());
@@ -226,6 +361,22 @@ class NetDbRenderer {
             SybilRenderer.renderSybilHTML(out, _context, sybils, sybil);
     }
 
+    /**
+     *  Special handling for 'O' cap
+     *  @param caps non-null
+     *  @since 0.9.38
+     */
+    private static boolean hasCap(RouterInfo ri, String caps) {
+        String ricaps = ri.getCapabilities();
+        if (caps.equals("O")) {
+            return ricaps.contains(caps) &&
+                   !ricaps.contains("P") &&
+                   !ricaps.contains("X");
+        } else {
+            return ricaps.contains(caps);
+        }
+    }
+
     /**
      *  @param debug @since 0.7.14 sort by distance from us, display
      *               median distance, and other stuff, useful when floodfill
@@ -369,24 +520,39 @@ class NetDbRenderer {
                         median = dist;
                 }
                 buf.append("&nbsp;&nbsp;<b>Distance: </b>").append(fmt.format(biLog2(dist)));
+                int type = ls.getType();
+                buf.append("&nbsp;&nbsp;<b>Type: </b>").append(type);
+                if (type != DatabaseEntry.KEY_TYPE_LEASESET) {
+                    LeaseSet2 ls2 = (LeaseSet2) ls;
+                    buf.append("&nbsp;&nbsp;<b>Unpublished? </b>").append(ls2.isUnpublished());
+                    boolean isOff = ls2.isOffline();
+                    buf.append("&nbsp;&nbsp;<b>Offline signed? </b>").append(isOff);
+                    if (isOff)
+                        buf.append("&nbsp;&nbsp;<b>Type: </b>").append(ls2.getTransientSigningKey().getType());
+                }
                 buf.append("</td></tr>\n<tr><td colspan=\"2\">");
                 //buf.append(dest.toBase32()).append("<br>");
                 buf.append("<b>Signature type:</b> ").append(dest.getSigningPublicKey().getType());
-                buf.append("&nbsp;&nbsp;<b>Encryption Key:</b> ").append(ls.getEncryptionKey().toBase64().substring(0, 20)).append("&hellip;");
+                if (type != DatabaseEntry.KEY_TYPE_META_LS2)
+                    buf.append("&nbsp;&nbsp;<b>Encryption Key:</b> ").append(ls.getEncryptionKey().toBase64().substring(0, 20)).append("&hellip;");
                 buf.append("</td></tr>\n<tr><td colspan=\"2\">");
                 buf.append("<b>Routing Key:</b> ").append(ls.getRoutingKey().toBase64());
                 buf.append("</td></tr>");
 
             }
             buf.append("<tr><td colspan=\"2\"><ul class=\"netdb_leases\">");
+            boolean isMeta = ls.getType() == DatabaseEntry.KEY_TYPE_META_LS2;
             for (int i = 0; i < ls.getLeaseCount(); i++) {
                 Lease lease = ls.getLease(i);
                 buf.append("<li><b>").append(_t("Lease")).append(' ').append(i + 1).append(":</b> <span class=\"netdb_gateway\" title=\"")
                    .append(_t("Gateway")).append("\"><img src=\"themes/console/images/info/gateway.png\" alt=\"")
                    .append(_t("Gateway")).append("\"></span> <span class=\"tunnel_peer\">");
                 buf.append(_context.commSystem().renderPeerHTML(lease.getGateway()));
-                buf.append("</span> <span class=\"netdb_tunnel\">").append(_t("Tunnel")).append(" <span class=\"tunnel_id\">")
-                   .append(lease.getTunnelId().getTunnelId()).append("</span></span> ");
+                buf.append("</span> ");
+                if (!isMeta) {
+                    buf.append("<span class=\"netdb_tunnel\">").append(_t("Tunnel")).append(" <span class=\"tunnel_id\">")
+                       .append(lease.getTunnelId().getTunnelId()).append("</span></span> ");
+                }
                 if (debug) {
                     long exl = lease.getEndDate().getTime() - now;
                     if (exl > 0)
@@ -424,27 +590,10 @@ class NetDbRenderer {
         out.flush();
     }
 
-    /**
-     * For debugging
-     * http://forums.sun.com/thread.jspa?threadID=597652
-     * @since 0.7.14
-     */
-    public static double biLog2(BigInteger a) {
-        int b = a.bitLength() - 1;
-        double c = 0;
-        double d = 0.5;
-        for (int i = b; i >= 0; --i) {
-             if (a.testBit(i))
-                 c += d;
-             d /= 2;
-        }
-        return b + c;
-    }
-
     /**
      *  @param mode 0: charts only; 1: full routerinfos; 2: abbreviated routerinfos
      */
-    public void renderStatusHTML(Writer out, int mode) throws IOException {
+    public void renderStatusHTML(Writer out, int pageSize, int page, int mode) throws IOException {
         if (!_context.netDb().isInitialized()) {
             out.write("<div id=\"notinitialized\">");
             out.write(_t("Not initialized"));
@@ -460,8 +609,29 @@ class NetDbRenderer {
         boolean showStats = full || shortStats;  // this means show the router infos
         Hash us = _context.routerHash();
 
+        Set<RouterInfo> routers = new TreeSet<RouterInfo>(new RouterInfoComparator());
+        routers.addAll(_context.netDb().getRouters());
+        int toSkip = pageSize * page;
+        boolean nextpg = routers.size() > toSkip + pageSize;
         StringBuilder buf = new StringBuilder(8192);
-        if (showStats) {
+        if (showStats && (page > 0 || nextpg)) {
+            buf.append("<div class=\"netdbnotfound\">");
+            if (page > 0) {
+                buf.append("<a href=\"/netdb?f=").append(mode).append("&amp;pg=").append(page)
+                   .append("&amp;ps=").append(pageSize).append("\">");
+                buf.append(_t("Previous Page"));
+                buf.append("</a>&nbsp;&nbsp;&nbsp;");
+            }
+            buf.append(_t("Page")).append(' ').append(page + 1);
+            if (nextpg) {
+                buf.append("&nbsp;&nbsp;&nbsp;<a href=\"/netdb?f=").append(mode).append("&amp;pg=").append(page + 2)
+                   .append("&amp;ps=").append(pageSize).append("\">");
+                buf.append(_t("Next Page"));
+                buf.append("</a>");
+            }
+            buf.append("</div>");
+        }
+        if (showStats && page == 0) {
             RouterInfo ourInfo = _context.router().getRouterInfo();
             renderRouterInfo(buf, ourInfo, true, true);
             out.write(buf.toString());
@@ -472,13 +642,22 @@ class NetDbRenderer {
         ObjectCounter<String> countries = new ObjectCounter<String>();
         int[] transportCount = new int[TNAMES.length];
 
-        Set<RouterInfo> routers = new TreeSet<RouterInfo>(new RouterInfoComparator());
-        routers.addAll(_context.netDb().getRouters());
+        int skipped = 0;
+        int written = 0;
+        boolean morePages = false;
         for (RouterInfo ri : routers) {
             Hash key = ri.getIdentity().getHash();
             boolean isUs = key.equals(us);
             if (!isUs) {
                 if (showStats) {
+                    if (skipped < toSkip) {
+                        skipped++;
+                        continue;
+                    }
+                    if (written++ >= pageSize) {
+                        morePages = true;
+                        break;
+                    }
                     renderRouterInfo(buf, ri, false, full);
                     out.write(buf.toString());
                     buf.setLength(0);
@@ -492,6 +671,21 @@ class NetDbRenderer {
                 transportCount[classifyTransports(ri)]++;
             }
         }
+        if (showStats && (page > 0 || morePages)) {
+            buf.append("<div class=\"netdbnotfound\">");
+            if (page > 0) {
+                buf.append("<a href=\"/netdb?f=").append(mode).append("&amp;pg=").append(page).append("&amp;ps=").append(pageSize).append("\">");
+                buf.append(_t("Previous Page"));
+                buf.append("</a>&nbsp;&nbsp;&nbsp;");
+            }
+            buf.append(_t("Page")).append(' ').append(page + 1);
+            if (morePages) {
+                buf.append("&nbsp;&nbsp;&nbsp;<a href=\"/netdb?f=").append(mode).append("&amp;pg=").append(page + 2).append("&amp;ps=").append(pageSize).append("\">");
+                buf.append(_t("Next Page"));
+                buf.append("</a>");
+            }
+            buf.append("</div>");
+        }
         long end = System.currentTimeMillis();
         if (log.shouldWarn())
             log.warn("part 1 took " + (end - start));
@@ -608,6 +802,27 @@ class NetDbRenderer {
         }
     }
 
+    /**
+     *  Sort by style, then host
+     *  @since 0.9.38
+     */
+    static class RAComparator implements Comparator<RouterAddress> {
+         private static final long serialVersionUID = 1L;
+
+         public int compare(RouterAddress l, RouterAddress r) {
+             int rv = l.getTransportStyle().compareTo(r.getTransportStyle());
+             if (rv != 0)
+                 return rv;
+             String lh = l.getHost();
+             String rh = r.getHost();
+             if (lh == null)
+                 return (rh == null) ? 0 : -1;
+             if (rh == null)
+                 return 1;
+             return lh.compareTo(rh);
+        }
+    }
+
     /**
      *  Be careful to use stripHTML for any displayed routerInfo data
      *  to prevent vulnerabilities
@@ -660,7 +875,13 @@ class NetDbRenderer {
         if (addrs.isEmpty()) {
             buf.append(_t("none"));
         } else {
-            for (RouterAddress addr : info.getAddresses()) {
+            if (addrs.size() > 1) {
+                // addrs is unmodifiable
+                List<RouterAddress> laddrs = new ArrayList<RouterAddress>(addrs);
+                Collections.sort(laddrs, new RAComparator());
+                addrs = laddrs;
+            }
+            for (RouterAddress addr : addrs) {
                 String style = addr.getTransportStyle();
                 buf.append("<br><b class=\"netdb_transport\">").append(DataHelper.stripHTML(style)).append(":</b>");
                 int cost = addr.getCost();
@@ -704,7 +925,7 @@ class NetDbRenderer {
         int rv = 0;
         for (RouterAddress addr : info.getAddresses()) {
             String style = addr.getTransportStyle();
-            if (style.equals("NTCP")) {
+            if (style.equals("NTCP") || style.equals("NTCP2")) {
                 rv |= NTCP;
             } else if (style.equals("SSU")) {
                 if (addr.getOption("iport0") != null)
