@@ -15,14 +15,19 @@ import net.i2p.data.Destination;
 /**
  * Interface for sending raw data to a SAM client
  */
-public interface SAMDatagramReceiver {
+interface SAMDatagramReceiver {
 
     /**
      * Send a byte array to a SAM client.
      *
+     * @param sender Destination
      * @param data Byte array to be received
+     * @param proto I2CP protocol
+     * @param fromPort I2CP from port
+     * @param toPort I2CP to port
+     * @throws IOException 
      */
-    public void receiveDatagramBytes(Destination sender, byte data[]) throws IOException;
+    public void receiveDatagramBytes(Destination sender, byte data[], int proto, int fromPort, int toPort) throws IOException;
 
     /**
      * Stop receiving data.
