@@ -323,7 +323,7 @@ public class I2PSocketManagerFull implements I2PSocketManager {
         boolean removed = _subsessions.remove(session);
         if (removed) {
             if (_log.shouldLog(Log.WARN))
-                _log.warn("Removeed subsession " + session);
+                _log.warn("Removed subsession " + session);
         } else {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Subsession not found to remove " + session);
@@ -350,7 +350,7 @@ public class I2PSocketManagerFull implements I2PSocketManager {
      * 
      * @return connected I2PSocket, or null through 0.9.16, non-null as of 0.9.17
      * @throws I2PException if session is closed
-     * @throws RouterRestartException (extends I2PException) if the router is apparently restarting, since 0.9.34
+     * @throws net.i2p.client.streaming.RouterRestartException (extends I2PException) if the router is apparently restarting, since 0.9.34
      * @throws ConnectException (since 0.9.17; I2PServerSocket interface always declared it)
      * @throws SocketTimeoutException if a timeout was previously set with setSoTimeout and the timeout has been reached.
      */
