@@ -1,9 +1,11 @@
 package net.i2p.crypto;
 
 import java.security.GeneralSecurityException;
-import java.security.Key;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.util.Arrays;
+import javax.crypto.Mac;
+import javax.crypto.SecretKey;
 import javax.crypto.spec.SecretKeySpec;
 
 import net.i2p.I2PAppContext;
@@ -19,7 +21,7 @@ import org.bouncycastle.oldcrypto.macs.I2PHMac;
  *
  * As of 0.9.12, uses javax.crypto.Mac.
  *
- * Deprecated, used only by Syndie.
+ * Warning - used by Syndie, don't break it.
  */
 public final class HMAC256Generator extends HMACGenerator {
 
@@ -33,6 +35,7 @@ public final class HMAC256Generator extends HMACGenerator {
      *  @throws UnsupportedOperationException since 0.9.12
      */
     @Override
+    @Deprecated
     protected I2PHMac acquire() {
         throw new UnsupportedOperationException();
     }
@@ -46,6 +49,7 @@ public final class HMAC256Generator extends HMACGenerator {
      *  @since 0.9.12 overrides HMACGenerator
      */
     @Override
+    @Deprecated
     public Hash calculate(SessionKey key, byte data[]) {
         throw new UnsupportedOperationException();
     }
@@ -60,9 +64,22 @@ public final class HMAC256Generator extends HMACGenerator {
      */
     @Override
     public void calculate(SessionKey key, byte data[], int offset, int length, byte target[], int targetOffset) {
+        calculate(key.getData(), data, offset, length, target, targetOffset);
+    }
+    
+    /**
+     *  Calculate the HMAC of the data with the given key.
+     *  Outputs 32 bytes to target starting at targetOffset.
+     *
+     *  @param key first 32 bytes used as the key
+     *  @throws UnsupportedOperationException if the JVM does not support it
+     *  @throws IllegalArgumentException for bad key or target too small
+     *  @since 0.9.38
+     */
+    public void calculate(byte[] key, byte data[], int offset, int length, byte target[], int targetOffset) {
         try {
-            javax.crypto.Mac mac = javax.crypto.Mac.getInstance("HmacSHA256");
-            Key keyObj = new SecretKeySpec(key.getData(), "HmacSHA256");
+            Mac mac = Mac.getInstance("HmacSHA256");
+            SecretKey keyObj = new HMACKey(key);
             mac.init(keyObj);
             mac.update(data, offset, length);
             mac.doFinal(target, targetOffset);
@@ -95,6 +112,24 @@ public final class HMAC256Generator extends HMACGenerator {
         return eq;
     }
 
+    /**
+     *  Like SecretKeySpec but doesn't copy the key in the construtor, for speed.
+     *  It still returns a copy in getEncoded(), because Mac relies
+     *  on that, doesn't work otherwise.
+     *  First 32 bytes are returned in getEncoded(), data may be longer.
+     *
+     *  @since 0.9.38
+     */
+    static final class HMACKey implements SecretKey {
+        private final byte[] _data;
+
+        public HMACKey(byte[] data) { _data = data; }
+
+        public String getAlgorithm() { return "HmacSHA256"; }
+        public byte[] getEncoded() { return Arrays.copyOf(_data, 32); }
+        public String getFormat() { return "RAW"; }
+    }
+
 /******
     private static class Sha256ForMAC extends Sha256Standalone implements Digest {
         public String getAlgorithmName() { return "sha256 for hmac"; }
@@ -128,18 +163,20 @@ public final class HMAC256Generator extends HMACGenerator {
      *
      */
 /****
+    private static final int LENGTH = 33;
+
     public static void main(String args[]) {
         I2PAppContext ctx = I2PAppContext.getGlobalContext();
         byte[] rand = new byte[32];
-        byte[] data = new byte[1500];
-        Key keyObj = new SecretKeySpec(rand, "HmacSHA256");
+        byte[] data = new byte[LENGTH];
+        SecretKey keyObj = null;
         SessionKey key = new SessionKey(rand);
 
         HMAC256Generator gen = new HMAC256Generator(I2PAppContext.getGlobalContext());
         byte[] result = new byte[32];
-        javax.crypto.Mac mac;
+        Mac mac;
         try {
-            mac = javax.crypto.Mac.getInstance("HmacSHA256");
+            mac = Mac.getInstance("HmacSHA256");
         } catch (NoSuchAlgorithmException e) {
             System.err.println("Fatal: " + e);
             return;
@@ -154,8 +191,7 @@ public final class HMAC256Generator extends HMACGenerator {
             byte[] keyBytes = keyObj.getEncoded();
             if (!DataHelper.eq(rand, keyBytes))
                 System.out.println("secret key in != out");
-            key = new SessionKey(rand);
-            gen.calculate(key, data, 0, data.length, result, 0);
+            gen.calculate(rand, data, 0, data.length, result, 0);
             try {
                 mac.init(keyObj);
             } catch (GeneralSecurityException e) {
@@ -163,17 +199,21 @@ public final class HMAC256Generator extends HMACGenerator {
                 return;
             }
             byte[] result2 = mac.doFinal(data);
-            if (!DataHelper.eq(result, result2))
-                throw new IllegalStateException();
+            if (!DataHelper.eq(result, result2)) {
+                throw new IllegalStateException("Mismatch on run " + i + ": result1:\n" +
+                                                net.i2p.util.HexDump.dump(result) +
+                                                "result1:\n" +
+                                                net.i2p.util.HexDump.dump(result2));
+            }
         }
 
         // real thing
         System.out.println("Passed");
         System.out.println("BC Test:");
-        RUNS = 500000;
+        RUNS = 1000000;
         long start = System.currentTimeMillis();
         for (int i = 0; i < RUNS; i++) {
-            gen.calculate(key, data, 0, data.length, result, 0);
+            gen.calculate(rand, data, 0, data.length, result, 0);
         }
         long time = System.currentTimeMillis() - start;
         System.out.println("Time for " + RUNS + " HMAC-SHA256 computations:");
