@@ -13,7 +13,6 @@ import java.math.BigInteger;
 import java.security.GeneralSecurityException;
 import java.security.InvalidKeyException;
 import java.security.KeyFactory;
-import java.security.KeyPair;
 import java.security.KeyPairGenerator;
 import java.security.MessageDigest;
 import java.security.ProviderException;
@@ -31,6 +30,8 @@ import java.util.Arrays;
 import java.util.ArrayList;
 import java.util.Collection;
 
+import com.southernstorm.noise.crypto.x25519.Curve25519;
+
 import net.i2p.I2PAppContext;
 import net.i2p.crypto.eddsa.EdDSAPrivateKey;
 import net.i2p.crypto.eddsa.EdDSAPublicKey;
@@ -173,20 +174,72 @@ public final class KeyGenerator {
         return keys;
     }
 
-    /** Convert a PrivateKey to its corresponding PublicKey
+    /**
+     *  Supports EncTypes
+     *  @since 0.9.38
+     */
+    public KeyPair generatePKIKeys(EncType type) {
+        PublicKey pub;
+        PrivateKey priv;
+        switch (type) {
+          case ELGAMAL_2048:
+            SimpleDataStructure[] keys = generatePKIKeys();
+            pub = (PublicKey) keys[0];
+            priv = (PrivateKey) keys[1];
+            break;
+
+          case ECIES_X25519:
+            byte[] bpriv = new byte[32];
+            do {
+                _context.random().nextBytes(bpriv);
+                // little endian, loop if too small
+                // worth doing?
+            } while (bpriv[31] == 0);
+            byte[] bpub = new byte[32];
+            Curve25519.eval(bpub, 0, bpriv, null);
+            pub = new PublicKey(type, bpub);
+            priv = new PrivateKey(type, bpriv);
+            break;
+
+          default:
+            throw new IllegalArgumentException("Unsupported algorithm");
+
+        }
+        return new KeyPair(pub, priv);
+    }
+
+    /**
+     * Convert a PrivateKey to its corresponding PublicKey.
+     * As of 0.9.38, supports EncTypes
+     *
      * @param priv PrivateKey object
      * @return the corresponding PublicKey object
      * @throws IllegalArgumentException on bad key
      */
     public static PublicKey getPublicKey(PrivateKey priv) {
-        BigInteger a = new NativeBigInteger(1, priv.toByteArray());
-        BigInteger aalpha = CryptoConstants.elgg.modPow(a, CryptoConstants.elgp);
-        PublicKey pub = new PublicKey();
-        try {
-            pub.setData(SigUtil.rectify(aalpha, PublicKey.KEYSIZE_BYTES));
-        } catch (InvalidKeyException ike) {
-            throw new IllegalArgumentException(ike);
+        EncType type = priv.getType();
+        byte[] data;
+        switch (type) {
+          case ELGAMAL_2048:
+            BigInteger a = new NativeBigInteger(1, priv.toByteArray());
+            BigInteger aalpha = CryptoConstants.elgg.modPow(a, CryptoConstants.elgp);
+            try {
+                data = SigUtil.rectify(aalpha, PublicKey.KEYSIZE_BYTES);
+            } catch (InvalidKeyException ike) {
+                throw new IllegalArgumentException(ike);
+            }
+            break;
+
+          case ECIES_X25519:
+            data = new byte[32];
+            Curve25519.eval(data, 0, priv.getData(), null);
+            break;
+
+          default:
+            throw new IllegalArgumentException("Unsupported algorithm");
+
         }
+        PublicKey pub = new PublicKey(type, data);
         return pub;
     }
 
@@ -234,7 +287,7 @@ public final class KeyGenerator {
     public SimpleDataStructure[] generateSigningKeys(SigType type) throws GeneralSecurityException {
         if (type == SigType.DSA_SHA1)
             return generateSigningKeys();
-        KeyPair kp;
+        java.security.KeyPair kp;
         if (type.getBaseAlgorithm() == SigAlgo.EdDSA) {
             net.i2p.crypto.eddsa.KeyPairGenerator kpg = new net.i2p.crypto.eddsa.KeyPairGenerator();
             kpg.initialize(type.getParams(), _context.random());
