@@ -7,6 +7,7 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.math.BigInteger;
 import java.security.GeneralSecurityException;
+import java.security.ProviderException;
 import java.security.MessageDigest;
 import java.security.PrivateKey;
 import java.security.PublicKey;
@@ -142,7 +143,12 @@ public final class SelfSignedGenerator {
         SigningPrivateKey priv = (SigningPrivateKey) keys[1];
         PublicKey jpub = SigUtil.toJavaKey(pub);
         PrivateKey jpriv = SigUtil.toJavaKey(priv);
-        return generate(jpub, jpriv, priv, type, cname, altNames, ou, o, l, st, c, validDays);
+        try {
+            return generate(jpub, jpriv, priv, type, cname, altNames, ou, o, l, st, c, validDays);
+        } catch (ProviderException pe) {
+            // PE is unchecked
+            throw new GeneralSecurityException(pe);
+        }
     }
 
     /**
@@ -184,6 +190,7 @@ public final class SelfSignedGenerator {
         }
         byte[] sigoid = getEncodedOIDSeq(oid);
 
+        // ProviderException thrown here
         byte[] tbs = genTBS(cname, altNames, ou, o, l, st, c, validDays, sigoid, jpub);
         int tbslen = tbs.length;
 
@@ -233,7 +240,7 @@ public final class SelfSignedGenerator {
         } catch (IllegalArgumentException iae) {
             throw new GeneralSecurityException("cert error", iae);
         }
-        X509CRL crl = generateCRL(cert, validDays, 1, sigoid, jpriv);
+        X509CRL crl = generateCRL(cert, validDays, 1, sigoid, priv);
 
         // some simple tests
         PublicKey cpub = cert.getPublicKey();
@@ -281,10 +288,7 @@ public final class SelfSignedGenerator {
      *  Generate a CRL for the given cert, signed with the given private key
      */
     private static X509CRL generateCRL(X509Certificate cert, int validDays, int crlNum,
-                                       byte[] sigoid, PrivateKey jpriv) throws GeneralSecurityException {
-
-        SigningPrivateKey priv = SigUtil.fromJavaKey(jpriv);
-
+                                       byte[] sigoid, SigningPrivateKey priv) throws GeneralSecurityException {
         byte[] tbs = genTBSCRL(cert, validDays, crlNum, sigoid);
         int tbslen = tbs.length;
 
@@ -382,6 +386,7 @@ public final class SelfSignedGenerator {
         byte[] validity = getValidity(validDays);
         byte[] subject = issuer;
 
+        // ProviderException thrown here
         byte[] pubbytes = jpub.getEncoded();
         byte[] extbytes = getExtensions(pubbytes, cname, altNames);
 
