@@ -11,8 +11,14 @@ import net.i2p.crypto.eddsa.spec.EdDSAParameterSpec;
  */
 public interface EdDSAKey {
     /**
-     * return a parameter specification representing the EdDSA domain
-     * parameters for the key.
+     * The reported key algorithm for all EdDSA keys
+     * @since 0.9.36
+     */
+    public String KEY_ALGORITHM = "EdDSA";
+
+    /**
+     * @return a parameter specification representing the EdDSA domain
+     *         parameters for the key.
      */
     public EdDSAParameterSpec getParams();
 }
