@@ -89,7 +89,8 @@ public class Certificate extends DataStructureImpl {
      * @since 0.8.3
      */
     public static Certificate create(InputStream in) throws DataFormatException, IOException {
-        int type = (int) DataHelper.readLong(in, 1);
+        // EOF will be thrown in next read
+        int type = in.read();
         int length = (int) DataHelper.readLong(in, 2);
         if (type == 0 && length == 0)
             return NULL_CERT;
@@ -161,7 +162,8 @@ public class Certificate extends DataStructureImpl {
     public void readBytes(InputStream in) throws DataFormatException, IOException {
         if (_type != 0 || _payload != null)
             throw new IllegalStateException("already set");
-        _type = (int) DataHelper.readLong(in, 1);
+        // EOF will be thrown in next read
+        _type = in.read();
         int length = (int) DataHelper.readLong(in, 2);
         if (length > 0) {
             _payload = new byte[length];
