@@ -11,6 +11,7 @@ package net.i2p.data;
 
 import java.util.Arrays;
 
+import net.i2p.crypto.Blinding;
 import net.i2p.crypto.KeyGenerator;
 import net.i2p.crypto.SigType;
 
@@ -88,20 +89,31 @@ public class SigningPrivateKey extends SimpleDataStructure {
         return KeyGenerator.getSigningPublicKey(this);
     }
 
+    /**
+     *  Only for SigType EdDSA_SHA512_Ed25519
+     *
+     *  @param h hash of secret data, same length as this key
+     *  @throws UnsupportedOperationException unless supported
+     *  @since 0.9.38
+     */
+    public SigningPrivateKey blind(SimpleDataStructure h) {
+        return Blinding.blind(this, h);
+    }
+
     /**
      *  @since 0.9.8
      */
     @Override
     public String toString() {
         StringBuilder buf = new StringBuilder(64);
-        buf.append('[').append(getClass().getSimpleName()).append(' ').append(_type).append(": ");
+        buf.append("[SigningPrivateKey ").append(_type).append(": ");
         int length = length();
         if (_data == null) {
             buf.append("null");
         } else if (length <= 32) {
             buf.append(toBase64());
         } else {
-            buf.append("size: ").append(Integer.toString(length));
+            buf.append("size: ").append(length);
         }
         buf.append(']');
         return buf.toString();
