@@ -14,6 +14,7 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.util.Arrays;
 
+import net.i2p.crypto.Blinding;
 import net.i2p.crypto.SigType;
 
 /**
@@ -196,17 +197,31 @@ public class SigningPublicKey extends SimpleDataStructure {
             out.write(_data, 0, KEYSIZE_BYTES);
     }
 
+    /**
+     *  Only for SigType EdDSA_SHA512_Ed25519
+     *
+     *  @param h hash of secret data, same length as this key
+     *  @throws UnsupportedOperationException unless supported
+     *  @since 0.9.38
+     */
+    public SigningPublicKey blind(SimpleDataStructure h) {
+        return Blinding.blind(this, h);
+    }
+
     /**
      *  @since 0.9.8
      */
     @Override
     public String toString() {
         StringBuilder buf = new StringBuilder(64);
-        buf.append('[').append(getClass().getSimpleName()).append(' ').append(_type).append(": ");
+        buf.append("[SigningPublicKey ").append((_type != null) ? _type.toString() : "unknown type").append(' ');
+        int length = length();
         if (_data == null) {
             buf.append("null");
+        } else if (length <= 32) {
+            buf.append(toBase64());
         } else {
-            buf.append("size: ").append(Integer.toString(length()));
+            buf.append("size: ").append(length);
         }
         buf.append(']');
         return buf.toString();
