@@ -51,11 +51,19 @@ public class ObjectCounter<K> implements Serializable {
     }
 
     /**
-     *  start over
+     *  Start over. Reset the count for all keys to zero.
      *  @since 0.7.11
      */
     public void clear() {
         this.map.clear();
     }
+
+    /**
+     *  Reset the count for this key to zero
+     *  @since 0.9.36
+     */
+    public void clear(K h) {
+        this.map.remove(h);
+    }
 }
 
