@@ -27,11 +27,14 @@ public abstract class SystemVersion {
 
     private static final boolean _isWin = System.getProperty("os.name").startsWith("Win");
     private static final boolean _isMac = System.getProperty("os.name").startsWith("Mac");
-    private static final boolean _isArm = System.getProperty("os.arch").startsWith("arm");
+    private static final boolean _isArm = System.getProperty("os.arch").startsWith("arm") ||
+                                          System.getProperty("os.arch").startsWith("aarch");
     private static final boolean _isX86 = System.getProperty("os.arch").contains("86") ||
                                           System.getProperty("os.arch").equals("amd64");
     private static final boolean _isGentoo = System.getProperty("os.version").contains("gentoo") ||
                                              System.getProperty("os.version").contains("hardened");  // Funtoo
+    // Could also check for java.vm.info = "interpreted mode"
+    private static final boolean _isZero = System.getProperty("java.vm.name").contains("Zero");
     private static final boolean _isAndroid;
     private static final boolean _isApache;
     private static final boolean _isGNU;
@@ -72,7 +75,7 @@ public abstract class SystemVersion {
         _isLinuxService = !_isWin && !_isMac && !_isAndroid &&
                           (DAEMON_USER.equals(System.getProperty("user.name")) ||
                            (_isGentoo && GENTOO_USER.equals(System.getProperty("user.name"))));
-        _isSlow = _isAndroid || _isApache || _isArm || _isGNU || getMaxMemory() < 48*1024*1024L;
+        _isSlow = _isAndroid || _isApache || _isArm || _isGNU || _isZero || getMaxMemory() < 48*1024*1024L;
 
         int sdk = 0;
         if (_isAndroid) {
@@ -162,6 +165,14 @@ public abstract class SystemVersion {
         return _isX86;
     }
 
+    /**
+     *  Is this a very slow interpreted mode VM?
+     *  @since 0.9.38
+     */
+    public static boolean isZeroVM() {
+        return _isZero;
+    }
+
     /**
      *  Our best guess on whether this is a slow architecture / OS / JVM,
      *  using some simple heuristics.
@@ -340,12 +351,12 @@ public abstract class SystemVersion {
         System.out.println("GNU      : " + isGNU());
         System.out.println("Linux Svc: " + isLinuxService());
         System.out.println("Mac      : " + isMac());
+        System.out.println("Max mem  : " + getMaxMemory());
         System.out.println("OpenJDK  : " + isOpenJDK());
         System.out.println("Slow     : " + isSlow());
         System.out.println("Windows  : " + isWindows());
         System.out.println("Wrapper  : " + hasWrapper());
         System.out.println("x86      : " + isX86());
-        System.out.println("Max mem  : " + getMaxMemory());
-
+        System.out.println("Zero JVM : " + isZeroVM());
     }
 }
