@@ -61,28 +61,8 @@ public class GarlicClove extends DataStructureImpl {
      *  @throws UnsupportedOperationException always
      */
     @Deprecated
-    public void readBytes(InputStream in) throws DataFormatException, IOException {
+    public void readBytes(InputStream in) {
         throw new UnsupportedOperationException();
-/****
-        _instructions = new DeliveryInstructions();
-        _instructions.readBytes(in);
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Read instructions: " + _instructions);
-        try {
-            _msg = _handler.readMessage(in);
-        } catch (I2NPMessageException ime) {
-            throw new DataFormatException("Unable to read the message from a garlic clove", ime);
-        }
-        _cloveId = DataHelper.readLong(in, 4);
-        _expiration = DataHelper.readDate(in);
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("CloveID read: " + _cloveId + " expiration read: " + _expiration);
-        //_certificate = new Certificate();
-        //_certificate.readBytes(in);
-        _certificate = Certificate.create(in);
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Read cert: " + _certificate);
-****/
     }
 
     /**
@@ -121,57 +101,8 @@ public class GarlicClove extends DataStructureImpl {
      *  @throws UnsupportedOperationException always
      */
     @Deprecated
-    public void writeBytes(OutputStream out) throws DataFormatException, IOException {
+    public void writeBytes(OutputStream out) {
         throw new UnsupportedOperationException();
-/****
-        StringBuilder error = null; 
-        if (_instructions == null) {
-            if (error == null) error = new StringBuilder();
-            error.append("No instructions ");
-        }
-        if (_msg == null) {
-            if (error == null) error = new StringBuilder();
-            error.append("No message ");
-        }
-        if (_cloveId < 0) {
-            if (error == null) error = new StringBuilder();
-            error.append("CloveID < 0 [").append(_cloveId).append("] ");
-        }
-        if (_expiration == null) {
-            if (error == null) error = new StringBuilder();
-            error.append("Expiration is null ");
-        }
-        if (_certificate == null) {
-            if (error == null) error = new StringBuilder();
-            error.append("Certificate is null ");
-        }
-        
-        if ( (error != null) && (error.length() > 0) )
-            throw new DataFormatException(error.toString());
-
-        _instructions.writeBytes(out);
-
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Wrote instructions: " + _instructions);
-        try {
-            byte m[] = _msg.toByteArray();
-            if (m == null)
-                throw new RuntimeException("foo, returned null");
-            if (m.length <= 0)
-                throw new RuntimeException("foo, returned 0 length");
-            out.write(m);
-        } catch (RuntimeException e) {
-            throw new DataFormatException("Unable to write the clove: " + _msg + " to " + out, e);
-        }
-        DataHelper.writeLong(out, 4, _cloveId);
-        DataHelper.writeDate(out, _expiration);
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("CloveID written: " + _cloveId + " expiration written: " 
-                       + _expiration);
-        _certificate.writeBytes(out);
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Written cert: " + _certificate);
-****/
     }
 
     /**
