@@ -36,7 +36,7 @@ public class OutNetMessage implements CDPQEntry {
     private final int _messageTypeId;
     /** cached message ID, for use after we discard the message */
     private final long _messageId;
-    private final long _messageSize;
+    private final int _messageSize;
     private final int _priority;
     private final long _expiration;
     private Job _onSend;
@@ -200,14 +200,17 @@ public class OutNetMessage implements CDPQEntry {
      *  @return the simple class name
      */
     public String getMessageType() {
-        I2NPMessage msg = _message;
-        return msg != null ? msg.getClass().getSimpleName() : "null";
+        return _message != null ? _message.getClass().getSimpleName() : "null";
     }
 
     public int getMessageTypeId() { return _messageTypeId; }
     public long getMessageId() { return _messageId; }
     
-    public long getMessageSize() {
+    /**
+     * How large the message is, including the full 16 byte header.
+     * Transports with different header sizes should adjust.
+     */
+    public int getMessageSize() {
         return _messageSize;
     }
     
@@ -371,12 +374,12 @@ public class OutNetMessage implements CDPQEntry {
             buf.append(getMessageType());
             buf.append(" ID ").append(_messageId);
         }
-        buf.append(" expiring on ").append(new Date(_expiration));
+        buf.append(" expiring ").append(new Date(_expiration));
         buf.append(" priority ").append(_priority);
         if (_failedTransports != null)
-            buf.append(" failed delivery on transports ").append(_failedTransports);
+            buf.append(" failed transports: ").append(_failedTransports);
         if (_target == null)
-            buf.append(" targetting no one in particular...");
+            buf.append(" (null target)");
         else
             buf.append(" targetting ").append(_target.getIdentity().getHash().toBase64());
         if (_onReply != null)
