@@ -180,6 +180,24 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
         return _lookupThrottler == null || _lookupThrottler.shouldThrottle(from, id);
     }
 
+    /**
+     *  If we are floodfill AND the key is not throttled,
+     *  flood it, otherwise don't.
+     *
+     *  @return if we did
+     *  @since 0.9.36 for NTCP2
+     */
+    public boolean floodConditional(DatabaseEntry ds) {
+        if (!floodfillEnabled())
+            return false;
+        if (shouldThrottleFlood(ds.getHash())) {
+            _context.statManager().addRateData("netDb.floodThrottled", 1);
+            return false;
+        }
+        flood(ds);
+        return true;
+    }
+
     /**
      *  Send to a subset of all floodfill peers.
      *  We do this to implement Kademlia within the floodfills, i.e.
@@ -216,6 +234,7 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
                 _log.info("Flooding the entry for " + key + " to " + i + " more, just before midnight");
         }
         int flooded = 0;
+        boolean isls2 = ds.isLeaseSet() && ds.getType() != DatabaseEntry.KEY_TYPE_LEASESET;
         for (int i = 0; i < peers.size(); i++) {
             Hash peer = peers.get(i);
             RouterInfo target = lookupRouterInfoLocally(peer);
@@ -228,6 +247,11 @@ public class FloodfillNetworkDatabaseFacade extends KademliaNetworkDatabaseFacad
                 continue;
             if (peer.equals(_context.routerHash()))
                 continue;
+            // min version checks
+            if (isls2 && !StoreJob.shouldStoreLS2To(target))
+                continue;
+            if (!StoreJob.shouldStoreTo(target))
+                continue;
             DatabaseStoreMessage msg = new DatabaseStoreMessage(_context);
             msg.setEntry(ds);
             OutNetMessage m = new OutNetMessage(_context, msg, _context.clock().now()+FLOOD_TIMEOUT, FLOOD_PRIORITY, target);
