@@ -33,7 +33,7 @@ import net.i2p.util.Log;
  * Receive DatabaseStoreMessage data and store it in the local net db
  *
  */
-public class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
+class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
     private final Log _log;
     private final DatabaseStoreMessage _message;
     private final RouterIdentity _from;
@@ -69,7 +69,8 @@ public class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
         RouterInfo prevNetDb = null;
         Hash key = _message.getKey();
         DatabaseEntry entry = _message.getEntry();
-        if (entry.getType() == DatabaseEntry.KEY_TYPE_LEASESET) {
+        int type = entry.getType();
+        if (DatabaseEntry.isLeaseSet(type)) {
             getContext().statManager().addRateData("netDb.storeLeaseSetHandled", 1);
             if (_log.shouldLog(Log.INFO))
                 _log.info("Handling dbStore of leaseset " + _message);
@@ -135,7 +136,7 @@ public class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
             } catch (IllegalArgumentException iae) {
                 invalidMessage = iae.getMessage();
             }
-        } else if (entry.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
+        } else if (type == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
             RouterInfo ri = (RouterInfo) entry;
             getContext().statManager().addRateData("netDb.storeRouterInfoHandled", 1);
             if (_log.shouldLog(Log.INFO))
@@ -181,7 +182,7 @@ public class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
             }
         } else {
             if (_log.shouldLog(Log.ERROR))
-                _log.error("Invalid DatabaseStoreMessage data type - " + entry.getType() 
+                _log.error("Invalid DatabaseStoreMessage data type - " + type
                            + ": " + _message);
             // don't ack or flood
             return;
@@ -251,7 +252,7 @@ public class HandleFloodfillDatabaseStoreMessageJob extends JobImpl {
         TunnelId replyTunnel = _message.getReplyTunnel();
         // A store of our own RI, only if we are not FF
         DatabaseStoreMessage msg2;
-        if ((getContext().netDb().floodfillEnabled() && !getContext().router().gracefulShutdownInProgress()) ||
+        if (getContext().netDb().floodfillEnabled() ||
             storedKey.equals(getContext().routerHash())) {
             // don't send our RI if the store was our RI (from PeerTestJob)
             msg2 = null;
