@@ -52,7 +52,7 @@ import net.i2p.util.Log;
  * Kademlia based version of the network database.
  * Never instantiated directly; see FloodfillNetworkDatabaseFacade.
  */
-public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
+public abstract class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
     protected final Log _log;
     private KBucketSet<Hash> _kb; // peer hashes sorted into kbuckets, but within kbuckets, unsorted
     private DataStore _ds; // hash to DataStructure mapping, persisted when necessary
@@ -136,6 +136,12 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
     private final static long ROUTER_INFO_EXPIRATION_FLOODFILL = 60*60*1000l;
     private final static long ROUTER_INFO_EXPIRATION_INTRODUCED = 45*60*1000l;
     
+    /**
+     * Don't let leaseSets go too far into the future 
+     */
+    private static final long MAX_LEASE_FUTURE = 15*60*1000;
+    private static final long MAX_META_LEASE_FUTURE = 65535*1000;
+    
     private final static long EXPLORE_JOB_DELAY = 10*60*1000l;
 
     /** this needs to be long enough to give us time to start up,
@@ -187,7 +193,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         return _initialized && _ds != null && _ds.isInitialized();
     }
 
-    protected PeerSelector createPeerSelector() { return new PeerSelector(_context); }
+    protected abstract PeerSelector createPeerSelector();
     public PeerSelector getPeerSelector() { return _peerSelector; }
     
     /** @since 0.9 */
@@ -438,7 +444,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         //return _ds.countLeaseSets();
         int rv = 0;
         for (DatabaseEntry ds : _ds.getEntries()) {
-            if (ds.getType() == DatabaseEntry.KEY_TYPE_LEASESET &&
+            if (ds.isLeaseSet() &&
                 ((LeaseSet)ds).getReceivedAsPublished())
                 rv++;
         }
@@ -466,13 +472,14 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         DatabaseEntry rv = _ds.get(key);
         if (rv == null)
             return null;
-        if (rv.getType() == DatabaseEntry.KEY_TYPE_LEASESET) {
+        int type = rv.getType();
+        if (DatabaseEntry.isLeaseSet(type)) {
             LeaseSet ls = (LeaseSet)rv;
             if (ls.isCurrent(Router.CLOCK_FUDGE_FACTOR))
                 return rv;
             else
                 fail(key);
-        } else if (rv.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
+        } else if (type == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
             try {
                 if (validate((RouterInfo)rv) == null)
                     return rv;
@@ -485,9 +492,9 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
     /**
      *  Not for use without validation
      *  @return RouterInfo, LeaseSet, or null, NOT validated
-     *  @since 0.9.9
+     *  @since 0.9.9, public since 0.9.38
      */
-    DatabaseEntry lookupLocallyWithoutValidation(Hash key) {
+    public DatabaseEntry lookupLocallyWithoutValidation(Hash key) {
         if (!_initialized)
             return null;
         return _ds.get(key);
@@ -518,8 +525,8 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
             if (onFindJob != null)
                 _context.jobQueue().addJob(onFindJob);
         } else if (isNegativeCached(key)) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn("Negative cached, not searching LS: " + key);
+            if (_log.shouldInfo())
+                _log.info("Negative cached, not searching LS: " + key);
             if (onFailedLookupJob != null)
                 _context.jobQueue().addJob(onFailedLookupJob);
         } else {
@@ -551,7 +558,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         if (!_initialized) return null;
         DatabaseEntry ds = _ds.get(key);
         if (ds != null) {
-            if (ds.getType() == DatabaseEntry.KEY_TYPE_LEASESET) {
+            if (ds.isLeaseSet()) {
                 LeaseSet ls = (LeaseSet)ds;
                 if (ls.isCurrent(Router.CLOCK_FUDGE_FACTOR)) {
                     return ls;
@@ -587,8 +594,8 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         if (d != null) {
             _context.jobQueue().addJob(onFinishedJob);
         } else if (isNegativeCached(key)) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn("Negative cached, not searching dest: " + key);
+            if (_log.shouldInfo())
+                _log.info("Negative cached, not searching dest: " + key);
         } else {
             search(key, onFinishedJob, onFinishedJob, timeoutMs, true, fromLocalDest);
         }
@@ -604,7 +611,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         if (!_initialized) return null;
         DatabaseEntry ds = _ds.get(key);
         if (ds != null) {
-            if (ds.getType() == DatabaseEntry.KEY_TYPE_LEASESET) {
+            if (ds.isLeaseSet()) {
                 LeaseSet ls = (LeaseSet)ds;
                 return ls.getDestination();
             }
@@ -624,13 +631,19 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
             if (onFailedLookupJob != null)
                 _context.jobQueue().addJob(onFailedLookupJob);
         } else if (isNegativeCached(key)) {
-            if (_log.shouldLog(Log.WARN))
-                _log.warn("Negative cached, not searching RI: " + key);
+            if (_log.shouldInfo())
+                _log.info("Negative cached, not searching RI: " + key);
         } else {
             search(key, onFindJob, onFailedLookupJob, timeoutMs, false);
         }
     }
     
+    /**
+     * This will return immediately with the result or null.
+     * However, this may still fire off a lookup if the RI is present but expired (and will return null).
+     * This may result in deadlocks.
+     * For true local only, use lookupLocallyWithoutValidation()
+     */
     public RouterInfo lookupRouterInfoLocally(Hash key) {
         if (!_initialized) return null;
         DatabaseEntry ds = _ds.get(key);
@@ -770,11 +783,6 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
     }
 ***/
     
-    /**
-     * Don't let leaseSets go 20 minutes into the future 
-     */
-    static final long MAX_LEASE_FUTURE = 20*60*1000;
-    
     /**
      * Determine whether this leaseSet will be accepted as valid and current
      * given what we know now.
@@ -817,7 +825,9 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
             return "Expired leaseSet for " + leaseSet.getDestination().toBase32()
                    + " expired " + DataHelper.formatDuration(age) + " ago";
         }
-        if (latest > now + (Router.CLOCK_FUDGE_FACTOR + MAX_LEASE_FUTURE)) {
+        if (latest > now + (Router.CLOCK_FUDGE_FACTOR + MAX_LEASE_FUTURE) &&
+            (leaseSet.getType() != DatabaseEntry.KEY_TYPE_META_LS2 ||
+             latest > now + (Router.CLOCK_FUDGE_FACTOR + MAX_META_LEASE_FUTURE))) {
             long age = latest - now;
             // let's not make this an error, it happens when peers have bad clocks
             if (_log.shouldLog(Log.WARN))
@@ -916,7 +926,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
             return "Invalid routerInfo signature";
         }
         if (routerInfo.getNetworkId() != _networkID){
-            _context.banlist().banlistRouter(key, "Not in our network");
+            _context.banlist().banlistRouterForever(key, "Not in our network: " + routerInfo.getNetworkId());
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Bad network: " + routerInfo);
             return "Not in our network";
@@ -1077,7 +1087,8 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
      */
     private void processStoreFailure(Hash h, DatabaseEntry entry) throws UnsupportedCryptoException {
         if (entry.getHash().equals(h)) {
-            if (entry.getType() == DatabaseEntry.KEY_TYPE_LEASESET) {
+            int etype = entry.getType();
+            if (DatabaseEntry.isLeaseSet(etype)) {
                 LeaseSet ls = (LeaseSet) entry;
                 Destination d = ls.getDestination();
                 Certificate c = d.getCertificate();
@@ -1094,7 +1105,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
                         }
                     } catch (DataFormatException dfe) {}
                 }
-            } else if (entry.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
+            } else if (etype == DatabaseEntry.KEY_TYPE_ROUTERINFO) {
                 RouterInfo ri = (RouterInfo) entry;
                 RouterIdentity id = ri.getIdentity();
                 Certificate c = id.getCertificate();
@@ -1237,7 +1248,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
         if (!_initialized) return null;
         Set<LeaseSet> leases = new HashSet<LeaseSet>();
         for (DatabaseEntry o : getDataStore().getEntries()) {
-            if (o.getType() == DatabaseEntry.KEY_TYPE_LEASESET)
+            if (o.isLeaseSet())
                 leases.add((LeaseSet)o);
         }
         return leases;
@@ -1281,14 +1292,7 @@ public class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacade {
     }
 
     /** unused (overridden in FNDF) */
-    public void sendStore(Hash key, DatabaseEntry ds, Job onSuccess, Job onFailure, long sendTimeout, Set<Hash> toIgnore) {
-        if ( (ds == null) || (key == null) ) {
-            if (onFailure != null) 
-                _context.jobQueue().addJob(onFailure);
-            return;
-        }
-        _context.jobQueue().addJob(new StoreJob(_context, this, key, ds, onSuccess, onFailure, sendTimeout, toIgnore));
-    }
+    public abstract void sendStore(Hash key, DatabaseEntry ds, Job onSuccess, Job onFailure, long sendTimeout, Set<Hash> toIgnore);
 
     /**
      *  Increment in the negative lookup cache
