@@ -20,7 +20,7 @@ import net.i2p.util.Log;
  * if the client is still connected.
  *
  */
-public class RepublishLeaseSetJob extends JobImpl {
+class RepublishLeaseSetJob extends JobImpl {
     private final Log _log;
     public final static long REPUBLISH_LEASESET_TIMEOUT = 60*1000;
     private final static int RETRY_DELAY = 20*1000;
