@@ -5,6 +5,7 @@ import java.net.InetAddress;
 import java.net.Inet6Address;
 import java.net.InetSocketAddress;
 import java.nio.ByteBuffer;
+import java.nio.Buffer;
 import java.nio.channels.CancelledKeyException;
 import java.nio.channels.ClosedChannelException;
 import java.nio.channels.ClosedSelectorException;
@@ -28,6 +29,7 @@ import net.i2p.data.router.RouterIdentity;
 import net.i2p.router.CommSystemFacade.Status;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.FIFOBandwidthLimiter;
+import net.i2p.util.TryCache;
 import net.i2p.util.Addresses;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.I2PThread;
@@ -53,7 +55,7 @@ class EventPumper implements Runnable {
     private final NTCPTransport _transport;
     private final ObjectCounter<ByteArray> _blockedIPs;
     private long _expireIdleWriteTime;
-    private boolean _useDirect;
+    private static final boolean _useDirect = false;
     
     /**
      *  This probably doesn't need to be bigger than the largest typical
@@ -63,13 +65,15 @@ class EventPumper implements Runnable {
     private static final int BUF_SIZE = 8*1024;
     private static final int MAX_CACHE_SIZE = 64;
 
-    /**
-     *  Read buffers. (write buffers use wrap())
-     *  Shared if there are multiple routers in the JVM
-     *  Note that if the routers have different PROP_DIRECT settings this will have a mix,
-     *  so don't do that.
-     */
-    private static final LinkedBlockingQueue<ByteBuffer> _bufCache = new LinkedBlockingQueue<ByteBuffer>(MAX_CACHE_SIZE);
+    private static class BufferFactory implements TryCache.ObjectFactory<ByteBuffer> {
+        public ByteBuffer newInstance() {
+            if (_useDirect) 
+                return ByteBuffer.allocateDirect(BUF_SIZE);
+            else
+                return ByteBuffer.allocate(BUF_SIZE);
+        }
+    }
+    
 
     /** 
      * every few seconds, iterate across all ntcp connections just to make sure
@@ -79,6 +83,7 @@ class EventPumper implements Runnable {
      * the time to iterate across them to check a few flags shouldn't be a problem.
      */
     private static final long FAILSAFE_ITERATION_FREQ = 2*1000l;
+    private static final int FAILSAFE_LOOP_COUNT = 512;
     private static final long SELECTOR_LOOP_DELAY = 200;
     private static final long BLOCKED_IP_FREQ = 3*60*1000;
 
@@ -90,9 +95,12 @@ class EventPumper implements Runnable {
     /**
      *  Do we use direct buffers for reading? Default false.
      *  NOT recommended as we don't keep good track of them so they will leak.
+     *
+     *  Unsupported, set _useDirect above.
+     *
      *  @see java.nio.ByteBuffer
      */
-    private static final String PROP_DIRECT = "i2np.ntcp.useDirectBuffers";
+    //private static final String PROP_DIRECT = "i2np.ntcp.useDirectBuffers";
 
     private static final int MIN_MINB = 4;
     private static final int MAX_MINB = 12;
@@ -101,6 +109,8 @@ class EventPumper implements Runnable {
         long maxMemory = SystemVersion.getMaxMemory();
         MIN_BUFS = (int) Math.max(MIN_MINB, Math.min(MAX_MINB, 1 + (maxMemory / (16*1024*1024))));
     }
+    
+    private static final TryCache<ByteBuffer> _bufferCache = new TryCache<>(new BufferFactory(), MIN_BUFS);
 
     public EventPumper(RouterContext ctx, NTCPTransport transport) {
         _context = ctx;
@@ -173,12 +183,12 @@ class EventPumper implements Runnable {
      */
     public void run() {
         int loopCount = 0;
+        int failsafeLoopCount = FAILSAFE_LOOP_COUNT;
         long lastFailsafeIteration = System.currentTimeMillis();
         long lastBlockedIPClear = lastFailsafeIteration;
         while (_alive && _selector.isOpen()) {
             try {
                 loopCount++;
-                runDelayedEvents();
 
                 try {
                     int count = _selector.select(SELECTOR_LOOP_DELAY);
@@ -189,6 +199,7 @@ class EventPumper implements Runnable {
                         // does clear() do anything useful?
                         selected.clear();
                     }
+                    runDelayedEvents();
                 } catch (ClosedSelectorException cse) {
                     continue;
                 } catch (IOException ioe) {
@@ -209,9 +220,13 @@ class EventPumper implements Runnable {
                     lastFailsafeIteration = now;
                     try {
                         Set<SelectionKey> all = _selector.keys();
-                        _context.statManager().addRateData("ntcp.pumperKeySetSize", all.size());
+                        int lastKeySetSize = all.size();
+                        _context.statManager().addRateData("ntcp.pumperKeySetSize", lastKeySetSize);
                         _context.statManager().addRateData("ntcp.pumperLoopsPerSecond", loopCount / (FAILSAFE_ITERATION_FREQ / 1000));
+                        // reset the failsafe loop counter,
+                        // and recalculate the max loops before failsafe sleep, based on number of keys
                         loopCount = 0;
+                        failsafeLoopCount = Math.max(FAILSAFE_LOOP_COUNT, 2 * lastKeySetSize);
                         
                         int failsafeWrites = 0;
                         int failsafeCloses = 0;
@@ -248,7 +263,7 @@ class EventPumper implements Runnable {
                                  */
                                 if ((!key.isValid()) &&
                                     (!((SocketChannel)key.channel()).isConnectionPending()) &&
-                                    con.getTimeSinceCreated() > 2 * NTCPTransport.ESTABLISH_TIMEOUT) {
+                                    con.getTimeSinceCreated(now) > 2 * NTCPTransport.ESTABLISH_TIMEOUT) {
                                     if (_log.shouldLog(Log.INFO))
                                         _log.info("Removing invalid key for " + con);
                                     // this will cancel the key, and it will then be removed from the keyset
@@ -279,10 +294,12 @@ class EventPumper implements Runnable {
                                     expire = _expireIdleWriteTime;
                                 }
 
-                                if ( con.getTimeSinceSend() > expire &&
-                                     con.getTimeSinceReceive() > expire) {
+                                if ( con.getTimeSinceSend(now) > expire &&
+                                     con.getTimeSinceReceive(now) > expire) {
                                     // we haven't sent or received anything in a really long time, so lets just close 'er up
-                                    con.close();
+                                    con.sendTerminationAndClose();
+                                    if (_log.shouldInfo())
+                                        _log.info("Failsafe or expire close for " + con);
                                     failsafeCloses++;
                                 }
                             } catch (CancelledKeyException cke) {
@@ -300,7 +317,8 @@ class EventPumper implements Runnable {
                     }
                 } else {
                     // another 100% CPU workaround 
-                    if ((loopCount % 512) == 511) {
+                    // TODO remove or only if we appear to be looping with no interest ops
+                    if ((loopCount % failsafeLoopCount) == failsafeLoopCount - 1) {
                         if (_log.shouldLog(Log.INFO))
                             _log.info("EventPumper throttle " + loopCount + " loops in " +
                                       (now - lastFailsafeIteration) + " ms");
@@ -314,15 +332,6 @@ class EventPumper implements Runnable {
                     _blockedIPs.clear();
                     lastBlockedIPClear = now;
                 }
-
-
-                // Clear the cache if the user changes the setting,
-                // so we can test the effect.
-                boolean newUseDirect = _context.getBooleanProperty(PROP_DIRECT);
-                if (_useDirect != newUseDirect) {
-                    _useDirect = newUseDirect;
-                    _bufCache.clear();
-                }
             } catch (RuntimeException re) {
                 _log.error("Error in the event pumper", re);
             }
@@ -360,7 +369,6 @@ class EventPumper implements Runnable {
         _wantsRead.clear();
         _wantsRegister.clear();
         _wantsWrite.clear();
-        _bufCache.clear();
     }
     
     /**
@@ -458,27 +466,11 @@ class EventPumper implements Runnable {
         _selector.wakeup();
     }
 
-    /**
-     *  How many to keep in reserve.
-     *  Shared if there are multiple routers in the JVM
-     */
-    private static int _numBufs = MIN_BUFS;
-    private static int __consecutiveExtra;
-
     /**
      *  High-frequency path in thread.
      */
     private ByteBuffer acquireBuf() {
-        ByteBuffer rv = _bufCache.poll();
-        // discard buffer if _useDirect setting changes
-        if (rv == null || rv.isDirect() != _useDirect) {
-            if (_useDirect)
-                rv = ByteBuffer.allocateDirect(BUF_SIZE);
-            else
-                rv = ByteBuffer.allocate(BUF_SIZE);
-            _numBufs++;
-        }
-        return rv;
+        return _bufferCache.acquire();
     }
     
     /**
@@ -493,21 +485,7 @@ class EventPumper implements Runnable {
             return;
         }
         buf.clear();
-        int extra = _bufCache.size();
-        boolean cached = extra < _numBufs;
-
-        // TODO always offer if direct?
-        if (cached) {
-            _bufCache.offer(buf);
-            if (extra > MIN_BUFS) {
-                __consecutiveExtra++;
-                if (__consecutiveExtra >= 20) {
-                    if (_numBufs > MIN_BUFS)
-                        _numBufs--;
-                    __consecutiveExtra = 0;
-                }
-            }
-        }
+        _bufferCache.release(buf);
     }
     
     private void processAccept(SelectionKey key) {
@@ -549,7 +527,9 @@ class EventPumper implements Runnable {
                 chan.socket().setKeepAlive(true);
 
             SelectionKey ckey = chan.register(_selector, SelectionKey.OP_READ);
-            new NTCPConnection(_context, _transport, chan, ckey);
+            NTCPConnection con = new NTCPConnection(_context, _transport, chan, ckey);
+            ckey.attach(con);
+            _transport.establishing(con);
         } catch (IOException ioe) {
             _log.error("Error accepting", ioe);
         }
@@ -565,6 +545,7 @@ class EventPumper implements Runnable {
             if (connected) {
                 if (shouldSetKeepAlive(chan))
                     chan.socket().setKeepAlive(true);
+                // key was already set when the channel was created, why do it again here?
                 con.setKey(key);
                 con.outboundConnected();
                 _context.statManager().addRateData("ntcp.connectSuccessful", 1);
@@ -607,73 +588,102 @@ class EventPumper implements Runnable {
      */
     private void processRead(SelectionKey key) {
         NTCPConnection con = (NTCPConnection)key.attachment();
-        ByteBuffer buf = acquireBuf();
+        ByteBuffer buf = null;
         try {
-            int read = con.getChannel().read(buf);
-            if (read < 0) {
-                if (con.isInbound() && con.getMessagesReceived() <= 0) {
-                    InetAddress addr = con.getChannel().socket().getInetAddress();
-                    int count;
-                    if (addr != null) {
-                        byte[] ip = addr.getAddress();
-                        ByteArray ba = new ByteArray(ip);
-                        count = _blockedIPs.increment(ba);
-                        if (_log.shouldLog(Log.WARN))
-                            _log.warn("Blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con);
+            while (true) {
+                buf = acquireBuf();
+                int read = 0;
+                int readThisTime;
+                int readCount = 0;
+                while ((readThisTime = con.getChannel().read(buf)) > 0)  {
+                    read += readThisTime;
+                    readCount++;
+                }
+                if (readThisTime < 0 && read == 0)
+                    read = readThisTime;
+                if (_log.shouldDebug())
+                    _log.debug("Read " + read + " bytes total in " + readCount + " times from " + con);
+                if (read < 0) {
+                    if (con.isInbound() && con.getMessagesReceived() <= 0) {
+                        InetAddress addr = con.getChannel().socket().getInetAddress();
+                        int count;
+                        if (addr != null) {
+                            byte[] ip = addr.getAddress();
+                            ByteArray ba = new ByteArray(ip);
+                            count = _blockedIPs.increment(ba);
+                            if (_log.shouldLog(Log.WARN))
+                                _log.warn("EOF on inbound before receiving any, blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con);
+                        } else {
+                            count = 1;
+                            if (_log.shouldLog(Log.WARN))
+                                _log.warn("EOF on inbound before receiving any: " + con);
+                        }
+                        _context.statManager().addRateData("ntcp.dropInboundNoMessage", count);
                     } else {
-                        count = 1;
-                        if (_log.shouldLog(Log.WARN))
-                            _log.warn("EOF on inbound before receiving any: " + con);
+                        if (_log.shouldLog(Log.DEBUG))
+                            _log.debug("EOF on " + con);
                     }
-                    _context.statManager().addRateData("ntcp.dropInboundNoMessage", count);
-                } else {
-                    if (_log.shouldLog(Log.DEBUG))
-                        _log.debug("EOF on " + con);
-                }
-                con.close();
-                releaseBuf(buf);
-            } else if (read == 0) {
-                // stay interested
-                //key.interestOps(key.interestOps() | SelectionKey.OP_READ);
-                releaseBuf(buf);
-                // workaround for channel stuck returning 0 all the time, causing 100% CPU
-                int consec = con.gotZeroRead();
-                if (consec >= 5) {
-                    _context.statManager().addRateData("ntcp.zeroReadDrop", 1);
-                    if (_log.shouldLog(Log.WARN))
-                        _log.warn("Fail safe zero read close " + con);
                     con.close();
-                } else {
-                    _context.statManager().addRateData("ntcp.zeroRead", consec);
-                    if (_log.shouldLog(Log.INFO))
-                        _log.info("nothing to read for " + con + ", but stay interested");
+                    releaseBuf(buf);
+                    break;
                 }
-            } else {
+                if (read == 0) {
+                    // stay interested
+                    //key.interestOps(key.interestOps() | SelectionKey.OP_READ);
+                    releaseBuf(buf);
+                    // workaround for channel stuck returning 0 all the time, causing 100% CPU
+                    int consec = con.gotZeroRead();
+                    if (consec >= 5) {
+                        _context.statManager().addRateData("ntcp.zeroReadDrop", 1);
+                        if (_log.shouldLog(Log.WARN))
+                            _log.warn("Fail safe zero read close " + con);
+                        con.close();
+                    } else {
+                        _context.statManager().addRateData("ntcp.zeroRead", consec);
+                        if (_log.shouldLog(Log.INFO))
+                            _log.info("nothing to read for " + con + ", but stay interested");
+                    }
+                    break;
+                }
+                // Process the data received
                 // clear counter for workaround above
                 con.clearZeroRead();
+                // go around again if we filled the buffer (so we can read more)
+                boolean keepReading = !buf.hasRemaining();
                 // ZERO COPY. The buffer will be returned in Reader.processRead()
-                buf.flip();
+                // not ByteBuffer to avoid Java 8/9 issues with flip()
+                ((Buffer)buf).flip();
                 FIFOBandwidthLimiter.Request req = _context.bandwidthLimiter().requestInbound(read, "NTCP read"); //con, buf);
                 if (req.getPendingRequested() > 0) {
                     // rare since we generally don't throttle inbound
                     key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
                     _context.statManager().addRateData("ntcp.queuedRecv", read);
                     con.queuedRecv(buf, req);
+                    break;
                 } else {
                     // stay interested
                     //key.interestOps(key.interestOps() | SelectionKey.OP_READ);
                     con.recv(buf);
                     _context.statManager().addRateData("ntcp.read", read);
+                    if (readThisTime < 0) {
+                        // EOF, we're done
+                        con.close();
+                        break;
+                    }
+                    if (!keepReading)
+                        break;
                 }
-            }
+            }  // while true
         } catch (CancelledKeyException cke) {
-            releaseBuf(buf);
+            if (buf != null)
+                releaseBuf(buf);
             if (_log.shouldLog(Log.WARN)) _log.warn("error reading on " + con, cke);
             con.close();
             _context.statManager().addRateData("ntcp.readError", 1);
         } catch (IOException ioe) {
             // common, esp. at outbound connect time
-            releaseBuf(buf);
+            if (buf != null)
+                releaseBuf(buf);
             if (con.isInbound() && con.getMessagesReceived() <= 0) {
                 InetAddress addr = con.getChannel().socket().getInetAddress();
                 int count;
@@ -682,11 +692,11 @@ class EventPumper implements Runnable {
                     ByteArray ba = new ByteArray(ip);
                     count = _blockedIPs.increment(ba);
                     if (_log.shouldLog(Log.WARN))
-                        _log.warn("Blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con);
+                        _log.warn("Blocking IP " + Addresses.toString(ip) + " with count " + count + ": " + con, ioe);
                 } else {
                     count = 1;
                     if (_log.shouldLog(Log.WARN))
-                        _log.warn("IOE on inbound before receiving any: " + con);
+                        _log.warn("IOE on inbound before receiving any: " + con, ioe);
                 }
                 _context.statManager().addRateData("ntcp.dropInboundNoMessage", count);
             } else {
@@ -706,7 +716,8 @@ class EventPumper implements Runnable {
             }
             con.close();
         } catch (NotYetConnectedException nyce) {
-            releaseBuf(buf);
+            if (buf != null)
+                releaseBuf(buf);
             // ???
             key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
             if (_log.shouldLog(Log.WARN))
