@@ -17,7 +17,7 @@ import net.i2p.data.Hash;
 import net.i2p.data.SessionKey;
 import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
-import net.i2p.router.util.CachedIteratorArrayList;
+import net.i2p.router.util.CachedIteratorCollection;
 import net.i2p.router.util.CoDelPriorityBlockingQueue;
 import net.i2p.router.util.PriBlockingQueue;
 import net.i2p.util.Log;
@@ -210,7 +210,7 @@ public class PeerState {
      *  Mostly messages that have been transmitted and are awaiting acknowledgement,
      *  although there could be some that have not been sent yet.
      */
-    private final List<OutboundMessageState> _outboundMessages;
+    private final CachedIteratorCollection<OutboundMessageState> _outboundMessages;
 
     /**
      *  Priority queue of messages that have not yet been sent.
@@ -370,7 +370,7 @@ public class PeerState {
         _rtt = INIT_RTT;
         _rttDeviation = _rtt;
         _inboundMessages = new HashMap<Long, InboundMessageState>(8);
-        _outboundMessages = new CachedIteratorArrayList<OutboundMessageState>(32);
+        _outboundMessages = new CachedIteratorCollection<OutboundMessageState>();
         //_outboundQueue = new CoDelPriorityBlockingQueue(ctx, "UDP-PeerState", 32);
         _outboundQueue = new PriBlockingQueue<OutboundMessageState>(ctx, "UDP-PeerState", 32);
         // all createRateStat() moved to EstablishmentManager
@@ -1560,7 +1560,7 @@ public class PeerState {
      *
      * @return number of active outbound messages remaining
      */
-    public int finishMessages() {
+    public int finishMessages(long now) {
         // short circuit, unsynchronized
         if (_outboundMessages.isEmpty())
             return _outboundQueue.size();
@@ -1582,7 +1582,7 @@ public class PeerState {
                         _retransmitter = null;
                     if (succeeded == null) succeeded = new ArrayList<OutboundMessageState>(4);
                     succeeded.add(state);
-                } else if (state.isExpired()) {
+                } else if (state.isExpired(now)) {
                     iter.remove();
                     if (_retransmitter == state)
                         _retransmitter = null;
@@ -1789,7 +1789,7 @@ public class PeerState {
         long now = _context.clock().now();
         if (state.getNextSendTime() <= now) {
             OutboundMessageState retrans = _retransmitter;
-            if ( (retrans != null) && ( (retrans.isExpired() || retrans.isComplete()) ) ) {
+            if ( (retrans != null) && ( (retrans.isExpired(now) || retrans.isComplete()) ) ) {
                 _retransmitter = null;
                 retrans = null;
 	    }
@@ -1904,7 +1904,6 @@ public class PeerState {
                 _context.statManager().addRateData("udp.sendConfirmFragments", state.getFragmentCount());
             _context.statManager().addRateData("udp.sendConfirmVolley", numSends);
             _transport.succeeded(state);
-            int numFragments = state.getFragmentCount();
             // this adjusts the rtt/rto/window/etc
             messageACKed(state.getMessageSize(), state.getLifetime(), numSends);
             //if (getSendWindowBytesRemaining() > 0)
@@ -1927,8 +1926,9 @@ public class PeerState {
         if (_dead)
             return false;
         
+        final long messageId = bitfield.getMessageId();
         if (bitfield.receivedComplete()) {
-            return acked(bitfield.getMessageId());
+            return acked(messageId);
         }
     
         OutboundMessageState state = null;
@@ -1936,7 +1936,7 @@ public class PeerState {
         synchronized (_outboundMessages) {
             for (Iterator<OutboundMessageState> iter = _outboundMessages.iterator(); iter.hasNext(); ) {
                 state = iter.next();
-                if (state.getMessageId() == bitfield.getMessageId()) {
+                if (state.getMessageId() == messageId) {
                     boolean complete = state.acked(bitfield);
                     if (complete) {
                         isComplete = true;
