@@ -12,6 +12,7 @@ import net.i2p.data.SessionKey;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.FIFOBandwidthLimiter;
 import net.i2p.router.util.CDQEntry;
+import net.i2p.util.TryCache;
 import net.i2p.util.Addresses;
 import net.i2p.util.Log;
 import net.i2p.util.SystemVersion;
@@ -45,8 +46,16 @@ class UDPPacket implements CDQEntry {
     // private boolean _isInbound;
     private FIFOBandwidthLimiter.Request _bandwidthRequest;
   
+    private static class PacketFactory implements TryCache.ObjectFactory<UDPPacket> {
+        static RouterContext context;
+        public UDPPacket newInstance() {
+            return new UDPPacket(context);
+        }
+    }
+    
     //  Warning - this mixes contexts in a multi-router JVM
-    private static final Queue<UDPPacket> _packetCache;
+    private static final TryCache<UDPPacket> _packetCache;
+    private static final TryCache.ObjectFactory<UDPPacket> _packetFactory;
     private static final boolean CACHE = true;
     private static final int MIN_CACHE_SIZE = 64;
     private static final int MAX_CACHE_SIZE = 256;
@@ -54,9 +63,11 @@ class UDPPacket implements CDQEntry {
         if (CACHE) {
             long maxMemory = SystemVersion.getMaxMemory();
             int csize = (int) Math.max(MIN_CACHE_SIZE, Math.min(MAX_CACHE_SIZE, maxMemory / (1024*1024)));
-            _packetCache = new LinkedBlockingQueue<UDPPacket>(csize);
+            _packetFactory = new PacketFactory();
+            _packetCache = new TryCache<>(_packetFactory, csize);
         } else {
             _packetCache = null;
+            _packetFactory = null;
         }
     }
     
@@ -396,23 +407,14 @@ class UDPPacket implements CDQEntry {
      *  @param inbound unused
      */
     public static UDPPacket acquire(RouterContext ctx, boolean inbound) {
-        UDPPacket rv = null;
+        UDPPacket rv;
         if (CACHE) {
-            rv = _packetCache.poll();
-            if (rv != null) {
-                synchronized(rv) {
-                    if (!rv._released) {
-                        Log log = rv._context.logManager().getLog(UDPPacket.class);
-                        log.error("Unreleased cached packet", new Exception());
-                        rv = null;
-                    } else {
-                        rv.init(ctx);
-                    }
-                }
-            }
-        }
-        if (rv == null)
+            PacketFactory.context = ctx;
+            rv = _packetCache.acquire();
+            rv.init(ctx);
+        } else {
             rv = new UDPPacket(ctx);
+        }
         return rv;
     }
 
@@ -440,7 +442,7 @@ class UDPPacket implements CDQEntry {
         }
         if (!CACHE)
             return;
-        _packetCache.offer(this);
+        _packetCache.release(this);
     }
     
     /**
@@ -448,8 +450,10 @@ class UDPPacket implements CDQEntry {
      *  @since 0.9.2
      */
     public static void clearCache() {
-        if (CACHE)
+        if (CACHE) {
+            PacketFactory.context = null;
             _packetCache.clear();
+        }
     }
 
     private synchronized void verifyNotReleased() {
